/*
 * $Header: /home/cvs/jakarta-commons-sandbox/jelly/src/java/org/apache/commons/jelly/tags/core/IncludeTag.java,v 1.6 2002/06/13 21:31:14 werken Exp $
 * $Revision: 1.6 $
 * $Date: 2002/06/13 21:31:14 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Commons", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 * 
 * $Id: IncludeTag.java,v 1.6 2002/06/13 21:31:14 werken Exp $
 */

package org.apache.commons.jelly.tags.core;

import java.net.URL;

import org.apache.commons.jelly.JellyContext;

import org.apache.commons.jelly.MissingAttributeException;

import org.apache.commons.jelly.Script;

import org.apache.commons.jelly.TagSupport;

import org.apache.commons.jelly.XMLOutput;

/** A tag which conditionally evaluates its body based on some condition
  *
  * @author <a href="mailto:jstrachan@apache.org">James Strachan</a>
  * @version $Revision: 1.6 $
  */

public class IncludeTag extends TagSupport {

    private String uri;

    private boolean shouldExport;
    private boolean shouldInherit;

    public IncludeTag() {
        this.shouldExport = false;
        this.shouldInherit = true;
    }

    public void setInherit(String inherit) {
        if ( "true".equals( inherit ) ) {
            this.shouldInherit = true;
        }
    }

    public void setExport(String export) {
        if ( "true".equals( export ) ) {
            this.shouldExport = true;
        }
    }

    public boolean getInherit() {
        return this.shouldInherit;
    }

    public boolean getExport() {
        return this.shouldExport;
    }

    // Tag interface

    //------------------------------------------------------------------------- 

    public void doTag(XMLOutput output) throws Exception {

        if (uri == null) {

            throw new MissingAttributeException( "uri" );

        }

        // we need to create a new JellyContext of the URI

        // take off the script name from the URL

        context.runScript(uri, output, getExport(), getInherit() );
    }

    // Properties

    //-------------------------------------------------------------------------                

    /** Sets the URI (relative URI or absolute URL) for the script to evaluate. */

    public void setUri(String uri) {

        this.uri = uri;

    }

}
